@@ -76,13 +76,15 @@ class ModelOptions(object):
     def __init__(self, custom_options_cls):
         self.props = {}
         self._prop_names_cache = None
-        
+
         # conditions for searching model instances within a storage and
         # validating them before saving:
-        self.must_have = None    
+        self.must_have = None
         self.must_not_have = None
-        
+
         self.label = None
+        self.label_plural = None
+        self.auto_label = None
 
         if custom_options_cls:    # the 'class Meta: ...' within model declaration
             custom_options = custom_options_cls.__dict__.copy()
@@ -98,6 +100,8 @@ class ModelOptions(object):
                 s = ', '.join(custom_options.keys())
                 raise TypeError("Invalid attribute(s) in 'class Meta': %s" % s)
 
+        self.referenced_by = {}    # { model: [attr_name1, attr_name2] }
+
     def add_prop(self, prop):
         self._prop_names_cache = None
         self.props[prop.attr_name] = prop
@@ -126,6 +130,13 @@ class ModelBase(type):
         attr_meta = attrs.pop('Meta', None)
         setattr(model, '_meta', ModelOptions(attr_meta))
 
+        # provide a nice label    TODO: allow to override, add plural form
+        model._meta.label = make_label(model.__name__)
+        model._meta.label_plural = model._meta.label + 's'
+        # this one is for URLs, automatically added relation descriptors, etc.,
+        # so it must not contain spaces and must not depend on i18n:
+        model._meta.auto_label = make_label(model.__name__).replace(' ', '_')
+
         # inherit model options from base classes
         # TODO: add extend=False to prevent inheritance (example: http://docs.djangoproject.com/en/dev/topics/forms/media/#extend)
         for base in bases:
@@ -133,6 +144,8 @@ class ModelBase(type):
                 model._meta.props.update(base._meta.props)  # TODO: check if this is secure
                 for name, prop in base._meta.props.iteritems():
                     model._meta.add_prop(prop)
+                    #if hasattr(prop, 'ref_model'):
+                    prop.contribute_to_model(model, name)
                 for name in IDENTITY_DICT_NAME, NEGATED_IDENTITY_DICT_NAME:
                     if hasattr(base._meta, name):
                         inherited = getattr(base._meta, name) or {}
@@ -147,13 +160,18 @@ class ModelBase(type):
             else:
                 setattr(model, attr_name, value)
 
+        # inherit relations so that backward relation descriptors are created
+        # for each model in the inheritance chain (e.g. Goal.tasks and
+        # Goal.delegated_tasks)
+        #for base in bases:
+        #    if hasattr(base, '_meta'):
+        #        print base._meta.props
+
         # fill some attrs from default search query    XXX  may be undesirable
         if model._meta.must_have:
             for k, v in model._meta.must_have.items():
-                setattr(model, k, v)
-
-        # provide a nice label    TODO: allow to override, add plural form
-        model._meta.label = make_label(model.__name__)
+                if not '__' in k:
+                    setattr(model, k, v)
 
         return model
 
@@ -184,6 +202,9 @@ class Model(object):
         >>> other_db = get_storage(backend='pymodels.backends.tokyo_tyrant',
         ...                        port=1983)
 
+        # let's make sure the storage is empty
+        >>> db.clear()
+
     See documentation on methods for more details.
 
     """
@@ -202,31 +223,28 @@ class Model(object):
 
         # saved instances are equal if they have the same key in same storage
         # even if their data differs
-        >>> note0 = Note(text='foo')
-        >>> note0.save(db)
-        'note_0'
-        >>> note0.text = 'quux'
-        >>> note0_retrieved = db.get(Note, note0.pk)
-        >>> note0 == note0_retrieved
+        >>> note1 = Note(text='foo')
+        >>> note1.save(db)
+        u'1'
+        >>> note1.text = 'quux'
+        >>> note1_retrieved = db.get(Note, note1.pk)
+        >>> note1 == note1_retrieved
         True
 
         # saved instances are different if they have different keys
-        >>> note1 = Note(text='bar')
-        >>> note1.save(db)
-        'note_1'
-        >>> note0 == note1
+        >>> note2 = Note(text='bar')
+        >>> note2.save(db)
+        u'2'
+        >>> note1 == note2
         False
 
         # saved instances are different if they have different storages
         # even if their keys are the same
-        >>> note1.save_as(note0.pk, other_db)
+        >>> note2.save_as(note1.pk, other_db)
         <Note bar>
-        >>> note0 == note1
+        >>> note1 == note2
         False
 
-        >>> Note.objects(db).delete()
-        >>> Note.objects(other_db).delete()
-
         """
         if not other:
             return False
@@ -289,12 +307,13 @@ class Model(object):
 
         Usage::
 
+            >>> db.clear()
             >>> Note.objects(db)
             []
             >>> Note(text="huh?").save(db)
-            'note_0'
+            u'1'
             >>> Note(text="hmmm...").save(db)
-            'note_1'
+            u'2'
             >>> Note.objects(db)
             [<Note huh?>, <Note hmmm...>]
             >>> Note.objects(db).where(text__contains='uh')
@@ -332,10 +351,11 @@ class Model(object):
         overlapping attributes -- ones that matter for both models). All other
         attributes are re-fetched from the database (if we know the key).
 
-        :Note: The document key is *preserved*. This means that the new instance
-        represents *the same document*, not a new one. Remember that models are
-        "views", and to "convert" a document does not mean copying; it can
-        however imply *adding* attributes to the existing document.
+        .. note:: The document key is *preserved*. This means that the new
+            instance represents *the same document*, not a new one. Remember
+            that models are "views", and to "convert" a document does not mean
+            copying; it can however imply *adding* attributes to the existing
+            document.
 
         Neither current instance nor the returned one are saved automatically.
         You will have to do it yourself.
@@ -412,7 +432,7 @@ class Model(object):
 
         Usage::
 
-            >>> note.objects(db).delete()
+            >>> db.clear()
             >>> note = Note(text="hello")   # just create the item
 
             # WRONG:
@@ -430,20 +450,20 @@ class Model(object):
 
             >>> new_key = note.save(db)                   # storage provided, key generated
             >>> new_key
-            'note_0'
+            u'1'
             >>> new_obj = note.save_as(storage=db)        # same as above
             >>> new_obj
             <Note hello>
             >>> new_obj.pk  # new key
-            'note_1'
+            u'2'
             >>> new_obj.text  # same data
             'hello'
             >>> new_key = note.save()                     # same storage, same key
             >>> new_key
-            'note_0'
+            u'1'
             >>> new_obj = note.save_as()                  # same storage, autogenerated new key
             >>> new_obj.pk
-            'note_2'
+            u'3'
             >>> new_obj = note.save_as('custom_key')      # same storage, key "123"
             >>> new_obj.pk
             'custom_key'
@@ -543,3 +563,13 @@ class Model(object):
         p = self._meta.props[name]
         value = getattr(self, name)
         return p.pre_save(value, storage) # will raise ValidationError if smth is wrong
+
+    def delete(self):
+        """
+        Deletes the object from the associated storage.
+        """
+        if not self._state.storage or not self._state.key:
+            raise ValueError('Cannot delete object: not associated with '
+                             'a storage and/or primary key is not defined.')
+        self._state.storage.delete(self._state.key)
+
